import {assert} from 'chai';
import Blackboard from '../../src/core/Blackboard';

/* BLACKBOARD ============================================================== */
suite('Core: Blackboard', function() {
    test('Basic Read & Write operations', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'this is some value');
        blackboard.set('var2', 999888);

        assert.equal(blackboard.get('var1'), 'this is some value');
        assert.equal(blackboard.get('var2'), 999888);
        assert.equal(blackboard.get('var3'), undefined);
    });

    test('Tree memory initialization', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'value', 'tree1');

        assert.isDefined(blackboard.get('var1', 'tree1'));
        assert.isDefined(blackboard.get('nodeMemory', 'tree1'));
        assert.isDefined(blackboard.get('openNodes', 'tree1'));
        assert.isDefined(blackboard.get('traversalCycle', 'tree1'));
    });

    test('Read & Write operations within Tree Scope', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'this is some value', 'tree 1');
        blackboard.set('var2', 999888, 'tree 2');

        assert.equal(blackboard.get('var1', 'tree 1'), 'this is some value');
        assert.equal(blackboard.get('var2', 'tree 2'), 999888);

        assert.equal(blackboard.get('var1', 'tree 2'), undefined);
        assert.equal(blackboard.get('var2', 'tree 1'), undefined);
    });

    test('Read & Write operations within Tree and Node Scopes', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'value 1', 'tree 1');
        blackboard.set('var2', 'value 2', 'tree 1', 'node 1');
        blackboard.set('var3', 'value 3', 'tree 1', 'node 2');
        blackboard.set('var4', 999888, 'tree 2');

        assert.equal(blackboard.get('var2', 'tree 1', 'node 1'), 'value 2');
        assert.equal(blackboard.get('var3', 'tree 1', 'node 2'), 'value 3');
        assert.equal(blackboard.get('var2', 'tree 1', 'node 2'), undefined);
        assert.equal(blackboard.get('var3', 'tree 1', 'node 1'), undefined);
        assert.equal(blackboard.get('var2', 'tree 1'), undefined);
        assert.equal(blackboard.get('var1', 'tree 1', 'node 1'), undefined);

        assert.equal(blackboard.get('var2', 'tree 2', 'node 1'), undefined);
    });

    test('Clear tree value.', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'value 1', 'tree 1');
        blackboard.set('var2', 'value 2', 'tree 1', 'node 1');
        blackboard.set('var3', 'value 3', 'tree 2', 'node 2');
        blackboard.set('var4', 999888, 'tree 2');
        blackboard.set('var5', 'value 5');

        blackboard.clear('tree 1');

        assert.equal(blackboard.get('var1', 'tree 1'), undefined);
        assert.equal(blackboard.get('var2', 'tree 1', 'node 1'), undefined);
        assert.equal(blackboard.get('var3', 'tree 2', 'node 2'), 'value 3');
        assert.equal(blackboard.get('var4', 'tree 2'), 999888);
        assert.equal(blackboard.get('var5'), 'value 5');
    });

    test('Clear all value.', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'value 1', 'tree 1');
        blackboard.set('var2', 'value 2', 'tree 1', 'node 1');
        blackboard.set('var3', 'value 3', 'tree 2', 'node 2');
        blackboard.set('var4', 999888, 'tree 2');
        blackboard.set('var5', 'value 5');

        blackboard.clear();

        assert.equal(blackboard.get('var1', 'tree 1'), undefined);
        assert.equal(blackboard.get('var2', 'tree 1', 'node 1'), undefined);
        assert.equal(blackboard.get('var3', 'tree 2', 'node 2'), undefined);
        assert.equal(blackboard.get('var4', 'tree 2'), undefined);
        assert.equal(blackboard.get('var5'), undefined);
    });

    test('Set value by kvObj.', function() {
        var blackboard = new Blackboard();

        blackboard.set('var1', 'value 1', 'tree 1');
        blackboard.set('var2', 'value 2', 'tree 1');

        assert.equal(blackboard.get('var2', 'tree 1'), 'value 2');

        blackboard.setFromObject({'var2': 'value 2', 'var3': 'value3'}, 'tree 1');

        assert.equal(blackboard.get('var1', 'tree 1'), 'value 1');
        assert.equal(blackboard.get('var2', 'tree 1'), 'value 2');

        blackboard.set('var1', 'value 1', 'tree 1', 'node 1');
        blackboard.setFromObject({'var1': 'value 1x', 'var2': 'value2'}, 'tree 1', 'node 1');
        assert.equal(blackboard.get('var1', 'tree 1', 'node 1'), 'value 1x');
    });
});
/* ========================================================================= */
